unit phComboBox;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Commctrl;

type

  TCBItem = class(TObject)
  private
  	FImageIndex: Integer;
    FImageIndent: Integer;
    FTextIndent: Integer;
  protected

  public

  published
  	property ImageIndex: Integer
    	read FImageIndex
      write FImageIndex;
    property ImageIndent: Integer
    	read FImageIndent
      write FImageIndent;
    property TextIndent: Integer
    	read FTextIndent
      write FTextIndent;

  end;

  TphComboBox = class(TCustomComboBox)
  private
    { Private declarations }
    FImages: TImageList;
    FImageChangeLink: TChangeLink;
    FImageList: TImageList;
    FHandle: HImageList;
    procedure SetImageList(Value: TImageList);
    procedure ImageListChange(Sender: TObject);
    procedure CNDrawItem(var Message: TWMDrawItem); message CN_DRAWITEM;
  protected
    { Protected declarations }
    property Canvas;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function MakeItem(ImageIndex, ImageIndent,
    	TextIndent: Integer): TCBItem;
  published
    { Published declarations }
    property Images: TImageList
    	read FImages
      write SetImageList;

    property ItemHeight;
    property Sorted;
    property Items;
    property Color;  
    property Ctl3D;  
    property DragCursor;  
    property DragMode;  
    property DropDownCount;  
    property Enabled;  
    property Font;  
    property ImeMode;  
    property ImeName;
    property MaxLength;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop; 
    property Visible;

    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnDropDown;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnStartDrag;
  end;

procedure Register;

implementation

constructor TphComboBox.Create(AOwner: TComponent);
begin
	inherited Create(AOwner);
  FImageChangeLink := TChangeLink.Create;
  FImageChangeLink.OnChange := ImageListChange;

  //Create ImageList with ILC_COLOR8 flag(halftone palette),
  //so that the color of the selected icon is same as Microsoft

  FImageList:= TImageList.CreateSize(16, 16);

  FHandle:= ImageList_Create( 16, 16,
  	ILC_COLOR8 or ILC_MASK, 4, 0 );

  FImageList.Handle:= FHandle;
  FImageList.ShareImages:= True;
  Style:= csOwnerDrawFixed;
end;

destructor TphComboBox.Destroy;
begin
	FImageChangeLink.Free;

  FImageList.Free;
  ImageList_Destroy(FHandle);
	inherited Destroy;
end;

function TphComboBox.MakeItem(ImageIndex, ImageIndent,
    	TextIndent: Integer): TCBItem;
var
	cbItem: TCBItem;
begin
	cbItem:= TCBItem.Create;
  cbItem.ImageIndex:= ImageIndex;
  cbItem.ImageIndent:= ImageIndent;
  cbItem.TextIndent:= TextIndent;

  Result:= cbItem;
end;

procedure TphComboBox.ImageListChange(Sender: TObject);
begin
	if HandleAllocated then
  begin
    if Sender = Images then
      FImageList.Handle:= TImageList(Sender).Handle;
  end;
end;

procedure TphComboBox.SetImageList(Value: TImageList);
begin
	if Images <> nil then
    Images.UnRegisterChanges(FImageChangeLink);

  FImages := Value;
  if Images <> nil then
  begin
    Images.RegisterChanges(FImageChangeLink);
    FImageList.Handle:= FImages.Handle;
  end
  else
  begin
  	FImageList.Handle:= FHandle;
  	FImageList.Clear;
  end;
  
end;
                                                    
procedure TphComboBox.CNDrawItem(var Message: TWMDrawItem);
var
  State: TOwnerDrawState;
  ImageWidth, ImageHeight: integer;
  cbItem: TCBITEM;
  cbFree: Boolean;
  str: string;

  rFocus: TRect;   //for focus
  rText: TRect;  //for text
  rImage: TRect;  //for image
  rFill: TRect;  //for FillRect
begin
  with Message.DrawItemStruct^ do
  begin
    State := TOwnerDrawState(WordRec(LongRec(itemState).Lo).Lo);
    Canvas.Handle := hDC;
    Canvas.Font := Font;
    Canvas.Brush := Brush;

    ImageWidth:= FImageList.Width;
    ImageHeight:= FImageList.Height;

    if (Integer(itemID) >= 0) and (odSelected in State) then
    begin
      Canvas.Brush.Color := clHighlight;
      Canvas.Font.Color := clHighlightText;
    end;

    //if user only add string to phcombobox
    //We need to create cbItem here
    cbFree:= False;
    cbItem:= TCBItem(Items.Objects[ItemID]);
    if cbItem = nil then
    begin
    	cbItem:= TCBItem.Create;
      cbItem.ImageIndex:= -1;
      cbItem.ImageIndent:= 0;
      cbItem.TextIndent:= 2;
      ImageWidth:= 0;
      ImageHeight:= 0;
      cbFree:= True;
    end;

    str:= Items[itemID];

    if Integer(itemID) >= 0 then
    begin
    	if (DroppedDown = True) and (rcItem.Top <> 3) then
      begin
      	//image rect --- Left
        rImage.Left:= rcItem.left + cbItem.ImageIndent;

        //TextIndent start from the right of image;
        rText.Left:= rImage.Left + ImageWidth + cbItem.TextIndent;

      end
      else
      begin
      	//Draw the item inside the combobox without indent,
      	//not the items in the droppeddown list
        rImage.Left:= rcItem.left;

        //if no image, no indent for the text
        if ImageWidth = 0 then
        	rText.Left:= rImage.Left + ImageWidth
        else
        	rText.Left:= rImage.Left + ImageWidth + cbItem.TextIndent;

      end;
      //Image Rect --- Top
      rImage.Top:= rcItem.Top + ((rcItem.Bottom - rcItem.Top) -
          	ImageHeight) div 2;

      //Text rect
      rText.Top:= rcITem.Top + ((rcItem.Bottom - rcItem.Top) -
        	Canvas.TextHeight(str)) div 2;

     	//FillRect
     	rFill:= rcItem;
     	rFill.Left:= rText.Left;
     	rFill.Right:= rFill.Left +
      	Canvas.TextWidth(str);

      //Focus Rect
      rFocus:= rFill;

      //Draw Text & Fill text rectangle
    	Canvas.FillRect(rFill);
      Canvas.TextOut(rText.Left, rText.Top, str);

      //Draw Image;
  		FImageList.DrawingStyle:= dsNormal;
  	 	FImageList.Draw(Canvas, rImage.Left, rImage.Top,
      	cbItem.ImageIndex);

    end;


    if (Integer(itemID) >= 0) and (odSelected in State) then
    begin
    	FImageList.BlendColor:= clDefault;
      FImageList.DrawingStyle:= dsSelected;
      FImageList.Draw(Canvas, rImage.Left, rImage.Top,
      	cbItem.ImageIndex);

    end;

    if (Integer(itemID) >= 0) and (odFocused in State) then
    begin

      if DroppedDown = false then
      begin
      	Canvas.Brush.Color:= clHighLight;
        Canvas.DrawFocusRect(rFocus);
      end
      else
      begin
      	//Draw special FocusRect when CBFolder is dropdown
        //This FocusRect is same as Microsoft's product
        Canvas.Brush.Color:= clWhite;
        Canvas.DrawFocusRect(rFocus);
        Canvas.Brush.Color:= clWindow;
        Canvas.DrawFocusRect(rFocus);
        Canvas.Brush.Color:= clWhite;
        Canvas.DrawFocusRect(rFocus);
        Canvas.Brush.Color:= clblack;
        Canvas.DrawFocusRect(rFocus);

      end;

    end;
    Canvas.Handle := 0;
    if cbFree = True then cbItem.Free;
  end;

end;

procedure Register;
begin
  RegisterComponents('phWare', [TphComboBox]);
end;

end.
